/*
    $Id: configfile.h,v 1.1.1.1 2001/08/21 10:04:43 samwise97 Exp $

    configfile.h -- simple class for reading/writing a configuration file
    Copyright (C) 2001 Iwan Mouwen <iwan@vvier.tudelft.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 *  I have searched for a inifile reader/writer class that I could add
 *  to lotrmap with little trouble, but I couldn't find a suitable candidate.
 *  So, as a nice exercise, I wrote my own. It's not complete yet (it cannot
 *  write files for example) but missing pieces can be added when they are needed.
 *  For now it does what it needs to do, so I'm happy with it :-)
 */

#ifndef CONFIGFILE_H
#define CONFIGFILE_H

#include <qfile.h>
#include <qtextstream.h>
#include <qlist.h>

//TODO add [add|get][Bool|Int|Float|...]

class Item
{
public:
	Item( const QString &item );
	Item( const QString &key, const QString &value );
	~Item();

	bool isKey( const QString &key );
	void setValue( const QString &value );
	bool isKeyValue() { return iskeyvalue; }

	/**
	 * only call get[Key|Value]() when you know key and value are valid!
         */
        QString getKey() { return *key; }
	QString getValue() { return *value; }

	QString getString();

private:
	bool iskeyvalue;
	QString * key, * value, * line;
};

class Section
{
public:
	Section( const QString &section );
	~Section();

	int getCount();
	QString getName();
	bool isSection( const QString &name );
	bool isKeyValue( const QString &line, QString &key, QString &value );
	bool findItem( const QString &key, Item **theitem );
	void addString( const QString &key, const QString &value );
	void addLine( const QString &line );
	QList<Item> * getList();

private:			
	QString name;
	QList<Item> itemList;

protected:
	void Clear();
};

class ConfigFile
{

public:
	ConfigFile( const QString &fileName );
	~ConfigFile();

	void readFile();
	//void writeFile();
	void closeFile();

	Section* addSection( const QString &section );
	bool findSection( const QString &section, Section **thesection, bool create=true );
	bool isSectionIdentifier( const QString &line, QString &name );
	void addLine( const QString &line );

	QString getString( const QString &section, const QString &key, const QString &Default );

	QList<Item> * getList( const QString &section );
private:
	QString fileName;
	QList<Section> sectionList;
        Section * currentSection;

protected:
	void Clear();
	void Init();
};


#endif
