/*
    $Id: lotrmap.cpp,v 1.1.1.1 2001/08/21 10:04:43 samwise97 Exp $

    lotrmap.cpp --
    Copyright (C) 2001 Iwan Mouwen <iwan@vvier.tudelft.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream.h>
#include <qdir.h>
#include <qaccel.h>
#include <qlayout.h>
//#include <qabstractlayout.h> //for debugging

#include <qhbox.h>
#include <qvbox.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qslider.h>
#include <qcheckbox.h>
#include <qfiledialog.h>

#include "lotrmap.h"
#include "configfile.h"


Lotrmap::Lotrmap()
{
	setCaption(tr("Lotrmap " VERSION));

	/* disable popupmenu for toolbar alignment */
        setDockMenuEnabled(false);		


	initActions();
	initMenuBar();
	initWidgets();

	maps.setAutoDelete( true );
	translations.setAutoDelete( true );
	
	theimage = new QImage;	

	places = new Places( true );
	found_places = new Places( false );

	readConfig();
	readMaps();
}

Lotrmap::~Lotrmap()
{
	delete theimage;
	delete places;
	delete found_places;
}

void Lotrmap::initActions()
{
	fileQuit = new QAction(tr("Exit"), tr("E&xit"), QAccel::stringToKey(tr("Ctrl+Q")), this);
	//fileQuit->setStatusTip(tr("Quits the application"));
	//fileQuit->setWhatsThis(tr("Exit\n\nQuits the application"));
	connect(fileQuit, SIGNAL(activated()), this, SLOT(slotFileQuit()));
	
	helpAboutApp = new QAction(tr("About"), tr("&About..."), 0, this);
	//helpAboutApp->setStatusTip(tr("About the application"));
	//helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
	connect(helpAboutApp, SIGNAL(activated()), this, SLOT(slotHelpAbout()));
}

void Lotrmap::initMenuBar()
{
        fileMenu=new QPopupMenu();
	mapsMenu = new QPopupMenu();

	fileMenu->insertItem( "&maps", mapsMenu );
	translationsMenu = new QPopupMenu();
	fileMenu->insertItem( "&translations", translationsMenu );

	fileMenu->insertSeparator();
	fileQuit->addTo(fileMenu);
		
	helpMenu=new QPopupMenu();
	helpAboutApp->addTo(helpMenu);


	menuBar()->insertItem(tr("&File"), fileMenu);
	menuBar()->insertSeparator();
	menuBar()->insertItem(tr("&Help"), helpMenu);
}

void Lotrmap::initWidgets()
{	
	/* a QGridLayout cannot be the CentralWidget (it doesn't inherit QWidget),
         * so we create a container widget that contains all other widgets and make this
         * widget the CentralWidget
         */

	QWidget *w = new QWidget( this );
	setCentralWidget( w );

	//TODO: fix runtime warning about layout
	// grid needs to be created before any other widget!!!
	// 3x3 with a 10 pixels width border and zero pixels between cells.
	QGridLayout *grid = new QGridLayout( w, 3, 3, 10, 0 );

	/* large image */
	QVBox *bbox = new QVBox( w );
	bbox->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	bigimage = new LImage ( false, bbox, "bigimage" );

	/* small image */
	QVBox *sbox = new QVBox( w );
	sbox->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	smallimage = new LImage ( true, sbox, "smallimage" );

	/* bottomright frame */
	QFrame *rboxframe = new QFrame( w );
	rboxframe->setFrameStyle( QFrame::Panel | QFrame::Sunken );

	// 5 pixels between children and border of rboxframe
	QVBoxLayout *rbox = new QVBoxLayout( rboxframe, 5 );
	
	QLabel *lselect = new QLabel( "Select:", rboxframe );	
	cb_select = new QComboBox( rboxframe );
	//cb_select->setMinimumWidth( 250 );
	connect( cb_select, SIGNAL(activated(int)), this, SLOT(slotCbSelectActivated(int)));

	QHBox *hbox1 = new QHBox( rboxframe );
	QLabel *lsearch = new QLabel( "Search:", hbox1 );
	QLabel *lfound = new QLabel( "0 found", hbox1 );
	hbox1->setStretchFactor( lsearch, 1 );

	cb_search = new QComboBox( true, rboxframe );
	cb_search->setEnabled( false );
	QLabel *lspeed = new QLabel( "Animation speed:", rboxframe );

	QSlider *speed = new QSlider( QSlider::Horizontal, rboxframe );
	speed->setEnabled( false );
	//TODO test these values
	speed->setMinValue( 1 );
	speed->setMaxValue( 50 );
	speed->setTickmarks( QSlider::Below );
	speed->setTickInterval( 10 );

	QHBox *hbox2 = new QHBox( rboxframe );
	QLabel *lslow = new QLabel( "slow", hbox2 );
	QLabel *lfast = new QLabel( "fast", hbox2 );
	hbox2->setStretchFactor( lslow, 1 );

	QCheckBox *animate = new QCheckBox( "Animate", rboxframe );
	animate->setEnabled( false );
	QCheckBox *crosshairs = new QCheckBox( "Crosshairs", rboxframe );
	crosshairs->setEnabled( false );

	rbox->addWidget( lselect );
	rbox->addWidget( cb_select );
	rbox->addWidget( hbox1 );
	rbox->addWidget( cb_search );
	rbox->addWidget( lspeed );
	rbox->addWidget( speed );
	rbox->addWidget( hbox2 );
	rbox->addWidget( animate );
	rbox->addWidget( crosshairs );
	rbox->addStretch();

/*
 +-------------+-----+----------+
 |   0, 0 ->   | 0,1 | 0,2      |
 |   0, 2      |     |          |
 |             +-----+----------+
 |             | 1,1 | 1,2      |
 |             +-----+----------+
 |             | 2,1 | 2,2      |
 |             |     |          |
 |             |     |          |
 +-------------+-----+----------+
*/	

	
	grid->setColStretch( 0, 1 );
	grid->setColStretch( 1, 0 );
	grid->setColStretch( 2, 0 );
	grid->addColSpacing( 1, 10 );

	grid->setRowStretch( 0, 0 );
	grid->setRowStretch( 1, 0 );
	grid->setRowStretch( 2, 1 );
	grid->addRowSpacing( 1, 20 );

	grid->addMultiCellWidget( bbox, 0, 2, 0, 0 );	
	grid->addWidget( sbox, 0, 2 );
	grid->addWidget( rboxframe, 2, 2 );
	


	bigimage->setSister(smallimage);
	smallimage->setSister(bigimage);
}

/*
void Lotrmap::slotFileOpen()
{
	bigimage->setUpdatesEnabled( false );
	smallimage->setUpdatesEnabled( false );
	
	QString filename = QFileDialog::getOpenFileName(0, "Image Files (*.png *.jpg *.bmp);;All files (*)", this);
	bigimage->setUpdatesEnabled( true );
	smallimage->setUpdatesEnabled( true );
	if (!filename.isEmpty()) {
		if ( theimage->load( filename ) ) {
			setCaption( filename );
			bigimage->setImage( theimage );
			smallimage->setImage( theimage );
		}
	}
}

*/


void Lotrmap::slotFileQuit()
{                                      	
	qApp->quit();
}

void Lotrmap::slotHelpAbout()
{
	QMessageBox::about(this,tr("About..."),
		tr("Lotrmap\nVersion " LOTRMAP_VERSION "\n(c) 1998-1999, 2001 by Iwan Mouwen") );
}

void Lotrmap::slotCbSelectActivated( int index )
{
	QPoint p = places->get( index );
	bigimage->gotoXY( p );
}

void Lotrmap::readConfig()
{
	//ConfigFile cf( "lotrmap.rc" );		
}

void Lotrmap::readMaps()
{
	for( QAction * a = maps.first(); a != 0; a = maps.next() ) {
		a->removeFrom(mapsMenu);	
	}
	maps.clear();

	QDir d( 0, "*.maprc", QDir::Name, QDir::Files );
	for (unsigned int i = 0; i < d.count(); i++ )
	{
		QString filename = d[i];
		int p1 = filename.findRev( ".maprc" );
		if ( p1 > 0 ) {
			ConfigFile map( d[i] );
			QString description( map.getString( "", "description", filename ) );
			if ( description.isEmpty() ) description = filename;

                 	QAction * a = new QAction(
				description, 				// Text
				description, 				// Menutext
				0,					// Accel
				this,					// Parent
				(const char *)filename.left(p1) );	// name
			connect(a, SIGNAL(activated()), this, SLOT(slotReadMap()));
			maps.append( a );
			a->addTo(mapsMenu);						
		}			
	}
}

/* don't call this as a regular function! */
void Lotrmap::slotReadMap()
{

	cb_select->clear();
	cb_search->clear();
	places->clear();
	found_places->clear();

	QAction * a = (QAction *)sender();
	QString filename = a->name();	
	openImageFile( filename );

	ConfigFile map( filename+".maprc" );

	currentMap = map.getString( "", "contains", "what_is_a_good_default_value_for_this?" );

	for( a = translations.first(); a != 0; a = translations.next() ) {
		a->removeFrom(translationsMenu);	
	}
	translations.clear();

	QDir d( 0, currentMap+"_*.dat", QDir::Name, QDir::Files );
	for (unsigned int i = 0; i < d.count(); i++ )
	{
		filename = d[i];
		int p1 = filename.findRev( "_" );
		int p2 = filename.findRev( ".dat" );
		if ( p2 - p1 > 1 ) {
			ConfigFile translation( filename );
			/* get the default language from the filename */
			QString language = filename.mid( p1+1, p2 - p1 -1 );
			/* then try to look up a better description */
			QString description = translation.getString( "", "language", language );

                 	QAction * a = new QAction(
				description, 			//Text
				description, 			//Menutext
				0,				// Accel
				this,				// Parent
				(const char *)language );	// name
			connect(a, SIGNAL(activated()), this, SLOT(slotReadTranslation()));
			translations.append( a );
			a->addTo(translationsMenu);						
		}			
	}
}

/* don't call this as a regular function! */
void Lotrmap::slotReadTranslation()
{
	QAction * a = (QAction *)sender();
	QString language = a->name();	
	openTranslationFile( language );
}



void Lotrmap::openImageFile( const QString &filename )
{
	if (!filename.isEmpty()) {
		if ( theimage->load( filename ) ) {
			setCaption( filename );
			bigimage->setImage( theimage );
			smallimage->setImage( theimage );
		}
	}	
}

void Lotrmap::openTranslationFile( const QString &language )
{
	cb_select->clear();
	cb_search->clear();
	places->clear();
	found_places->clear();
	currentLanguage = language;

	ConfigFile f( currentMap+"_"+language+".dat" );		

	QList<Item> * list = f.getList("data");

	for ( Item * i = list->first() ; i != 0; i =list->next() ) {
		places->add( i->getKey(), i->getValue() );
	}
	cb_select->insertStringList( places->getList() );

	delete list;
}
