/*
    $Id: place.cpp,v 1.1.1.1 2001/08/21 10:04:43 samwise97 Exp $

    place.cpp --
    Copyright (C) 2001 Iwan Mouwen <iwan@vvier.tudelft.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "place.h"

#define __DEBUG(c) c


Place::Place( const QString &name, const QPoint &point )
{
	this->name = name;
	this->point = point;
}

Place::~Place()
{
}

bool Place::contains( const QString &name ) const
{
	return( this->name.contains( name ) );
}

Places::Places( bool AutoDelete = false )
{
	placeList.setAutoDelete( AutoDelete );
	placeList.clear();
}

Places::~Places()
{

}

void Places::add( const Place * place )
{
	placeList.inSort( place );
}

void Places::add( const QString &name, const QPoint &point )
{
	Place * p = new Place( name, point );
	add( p );
}

void Places::add( const QString &name, const QString &point )
{
	int pos = point.find(",");
	QString sx = point.left( pos );
	QString sy = point.mid( pos+1 );
	bool ok;
	int x = sx.toInt( &ok );
	if (ok) {
		int y = sy.toInt( &ok );
		if (ok) {
			add( name, QPoint(x, y) ); 	
		}
	}
}

QPoint Places::get( unsigned int index )
{
	if ( index < placeList.count() ) {
		Place * p = placeList.at(index);
		return p->getPoint();
	}
	else
		return QPoint( 0, 0 );
}

Places Places::grep( const QString &name )
{
	Places result;
	Place * p;
	for( p = placeList.first(); p != 0; p = placeList.next() ) {
		if ( p->contains( name ) )
			result.add( p );
	}
	return result;
}

QStringList Places::getList()
{
	QStringList result;
	Place * p;
	for( p = placeList.first(); p != 0; p = placeList.next() ) {
		result.append( p->getName() );
	}
	return result;	                           	
}
